/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for GeneClust user interface
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  GeneClust
 *
 *============================================================================*/

/*
 * @(#)GeneClust.java    0.95 2001/02/02
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.geneclust;

import java.lang.System;     // workaround to force @see hyperlink
import java.io.File;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ResourceBundle;


/**
 * A Java application for GeneClust research.
 *
 * @version 0.23 2000/12/01
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class GeneClust
    implements GeneClustKeys
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public GeneClust()
    {
        instance = this;

        String windowTitle = getBundleString(APPLICATION_WINDOW_TITLE);
        geneclustFrame = new GeneClustFrame(windowTitle);
        geneclustPanel = geneclustFrame.getUserInterface();
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Displays the user interface
     */
    public void display()
    {
        geneclustFrame.setVisible(true);
    }

    /**
     * Displays error message to user
     *
     * @param message  error message to be displayed
     */
    public void displayError(String message)
    {
        System.err.println(message);
        if (geneclustPanel != null)
            geneclustPanel.displayError(message);
    }

    /**
     * Returns the user interface panel
     *
     * @return  instance of <code>GeneClustPanel</code> object
     */
    public GeneClustPanel getUserInterface()
    {
        return geneclustPanel;
    }

    // =========================================================================
    // CLASS INTERFACE (Public)
    // =========================================================================

    /**
     * Causes termination of the Java Virtual Machine. The argument serves
     * as a determination of what the exit status code should be. This method
     * never returns normally.
     *
     * @param reason  if non-zero, abnormal termination is indicated;
     *                otherwise, successful termination is indicated.
     * @see java.lang.System#exit(int)
     */
    public static void terminate(int reason)
    {
        final int EXIT_SUCCESS = 0;
        final int EXIT_FAILURE = 1;

        System.exit((reason == 0) ? EXIT_SUCCESS : EXIT_FAILURE);
    }

    // =========================================================================
    // CLASS INTERFACE (Package)
    // =========================================================================

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message
     */
    static String getBundleString(String propertyKey)
    {
        return (bundle == null) ? null : bundle.getString(propertyKey);
    }

    /**
     * Factory method that returns reference for this object
     *
     * @return reference for this instance
     */
    static GeneClust getInstance()
    {
        return instance;
    }

    /**
     * Returns the name of this package.
     *
     * @return string specifying package name using Java language dot notation
     * @see Package#getName
     */
    static String getPackageName()
    {
        return GeneClust.class.getPackage().getName();
    }

    /**
     * Finds a resource with a given name. This method returns null if no
     * resource with this name is found. The rules for searching resources
     * associated with a given class are implemented by the defining class
     * loader of the class.
     *
     * @param resourceName  string specifying name of the desired resource
     * @return              url of requested resource
     * @see Class#getResource
     */
    static URL getResource(String resourceName)
    {
        return GeneClust.class.getResource(resourceName);
    }

    // =========================================================================
    // CLASS INTERFACE (Private)
    // =========================================================================

    /**
     * Determines if the version of the Java Virtual Machine is
     * adequate to run this application.
     *
     * @return if true, okay to proceed;
     *         otherwise, terminate due to need for more current JVM.
     */
    private static boolean isMinimumJVM()
    {
        final String vers = System.getProperty("java.version");
        final String minimumJavaVirtualMachine = "1.2.0";

        if (vers.compareTo(minimumJavaVirtualMachine) < 0)
        {
            String formatStr = getBundleString(ERR_JVM_PREREQUISITE);
            Object[] args = { minimumJavaVirtualMachine };
            MessageFormat mf = new MessageFormat(formatStr);
            final String errorMessage = mf.format(args);

            System.err.println(errorMessage);
            return false;
        }

        return true;
    }

    /**
     * Displays the command line usage of this application. Invoked upon
     * startup when invalid arguments are received from the command line.
     */
    private static void usage()
    {
        String className = getInstance().getClass().getName();

        String formatStr = getBundleString(APPLICATION_USAGE);
        Object[] args = { className };
        MessageFormat mf = new MessageFormat(formatStr);
        final String usageMessage = mf.format(args);

        System.out.println(usageMessage);
    }

    // =========================================================================
    // MAIN
    // =========================================================================

    /**
     * Main entry-point for the GeneClust class
     *
     * @param argv  string array containing command line arguments
     * @see
     * <a href="http://odin.mdacc.tmc.edu/~roebuck/geneclust/GeneClust.html">
     * GeneClust User Guide</a>
     */
    public static void main(String[] argv)
    {
        System.out.println(getBundleString(INFO_STARTUP));

        if (isMinimumJVM() == false)
            terminate(-1);

        if (argv.length > 1)
        {
            usage();
            terminate(-1);
        }

        // Display splash screen
        GCSplashScreen splash = new GCSplashScreen();
        splash.display();

        // Display user interface
        GeneClust geneclust = new GeneClust();
        geneclust.display();

        if (argv.length == 1)
            geneclust.getUserInterface().loadConfiguration(new File(argv [0]));
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final ResourceBundle bundle;
    private static GeneClust instance = null;

    static
    {
        final String propertyFile = GeneClust.class.getName();
        bundle = ResourceBundle.getBundle(propertyFile);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    private GeneClustFrame geneclustFrame;
    private GeneClustPanel geneclustPanel;
}

