/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for GeneClust user interface (GeneClustPanel)
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  GeneClust
 *
 *============================================================================*/

/*
 * @(#)GeneClustPanel.java    0.95 2001/02/02
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.geneclust;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import javax.accessibility.AccessibleContext;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.KeyStroke;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import javax.swing.border.TitledBorder;
import org.mdanderson.biostat.util.AutoSelectEventHandler;
import org.mdanderson.biostat.util.GuiUtil;
import org.mdanderson.biostat.util.InetProtocolHandler;
import org.mdanderson.biostat.util.NumericTextField;
import org.mdanderson.biostat.util.PathnameTextField;
import org.mdanderson.biostat.util.StatusEvent;
import org.mdanderson.biostat.util.StatusListener;
import org.mdanderson.biostat.util.VerticalFlowLayout;


/**
 * GeneClustPanel - Implements the user interface of the application.
 *
 * @version 0.95 2001/02/20
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class GeneClustPanel extends JPanel
    implements GeneClustKeys
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor
     *
     * @param frame  parent component
     */
    public GeneClustPanel(JFrame frame)
    {
        super();
        setName("GeneClustPanel");

        this.frame = frame;

        createMenuBar();

        setLayout(new BorderLayout());
        add(createClientPanel(), BorderLayout.CENTER);

        contentPane = frame.getContentPane();

        addNotify();
    }

    // =========================================================================
    // PUBLIC INTERFACE (Accessors)
    // =========================================================================

    /**
     * Returns boolean that tells whether original matrix should be
     * written to disk or not. Only really needed for demo mode since
     * there is otherwise no means of seeing what got 'created' based
     * on the interface settings on the demo tab. In raw data mode,
     * the data file already exists.
     *
     * @return true or false
     */
    public boolean isWriteMatrix()
    {
        return isDemoEnabled();
    }

    /**
     * Returns boolean that tells whether to display graph of
     * hierarchial clustering of genes should be performed or not.
     *
     * @return true or false
     */
    public boolean isDisplayGenesGraph()
    {
        return displayGenesGraphCheckBox.isSelected();
    }

    /**
     * Returns boolean that tells whether to display graph of
     * hierarchial clustering of samples should be performed or not.
     *
     * @return true or false
     */
    public boolean isDisplaySamplesGraph()
    {
        return displaySamplesGraphCheckBox.isSelected();
    }

    /**
     * Returns boolean that tells whether to display graph of
     * hierarchial clustering of samples should be performed or not.
     *
     * @return true or false
     */
    public boolean isDisplayDataMatrix()
    {
        return displayDataMatrixCheckBox.isSelected();
    }

    /**
     * Returns boolean that tells whether to display graph of
     * hierarchial clustering of samples should be performed or not.
     *
     * @return true or false
     */
    public boolean isDisplayShaving()
    {
        return displayShavingCheckBox.isSelected();
    }

    /**
     * Returns boolean that tells whether supervised shaving should be
     * performed or not.
     *
     * @return true or false
     */
    public boolean isPerformSupervised()
    {
        return getPercentSupervision() > 0;
    }

    /**
     * Returns the percent supervision
     *
     * @return integer in range 1..100
     */
    public int getPercentSupervision()
    {
        return Integer.parseInt(percentSupervisionTextField.getText().trim());
    }

    /**
     * Returns the string representing the classification filename
     *
     * @return string describing the classification filename
     */
    public String getClassificationFileName()
    {
        return classificationFileNameTextField.getText().trim();
    }

    /**
     * Returns the number of clusters to extract
     *
     * @return integer in range 1..128
     */
    public int getNumClustersToExtract()
    {
        return Integer.parseInt(numExtClustersTextField.getText().trim());
    }

    /**
     * Returns the number of permutations to find optimal
     *
     * @return integer in range 1..100
     */
    public int getNumPermutations()
    {
        return Integer.parseInt(numPermutationsTextField.getText().trim());
    }

    /**
     * Returns the gap tolerance
     *
     * @return integer in range 1..100
     */
    public int getGapTolerance()
    {
        return Integer.parseInt(gapToleranceTextField.getText().trim());
    }

    /**
     * Returns the percent to shave
     *
     * @return integer in range 1..99
     */
    public int getPercentToShave()
    {
        return Integer.parseInt(percentToShaveTextField.getText().trim());
    }

    /**
     * Returns the number of rows in the gene expression matrix (genes)
     *
     * @return integer in range 1..32767
     */
    public int getNumRows()
    {
        return Integer.parseInt(numRowsTextField.getText().trim());
    }

    /**
     * Returns the number of samples in the gene expression matrix (cell lines)
     *
     * @return integer in range 1..32767
     */
    public int getNumColumns()
    {
        return Integer.parseInt(numColumnsTextField.getText().trim());
    }

    /**
     * Returns the maximum value of signal used when generating matrix
     *
     * @return double in range 1.0..20.0
     */
    public double getAmplitude()
    {
        return Double.parseDouble(amplitudeTextField.getText().trim());
    }

    /**
     * Returns the string representing the data model
     *
     * @param removeWhitespace  if true, remove any whitespace from entry
     * @return string describing the data model
     */
    public String getModel(boolean removeWhitespace)
    {
// :TODO: This method doesn't handle i18n in its current form.
// Some manner of HashMap needed to separate between list choices
// and their internal representation is required.

        String selectedItem = (String) modelComboBox.getSelectedItem();

        if (removeWhitespace == true)
        {
            int idx = selectedItem.lastIndexOf(' ', selectedItem.length());

            // Remove any whitespace from item before returning
            if (idx != -1)
            {
                StringBuffer buff = new StringBuffer(selectedItem);

                do
                {
                    buff.deleteCharAt(idx);
                    idx = selectedItem.lastIndexOf(' ', idx-1);
                } while (idx != -1);

                selectedItem = buff.toString();
            }
        }

        return selectedItem;
    }

    /**
     * Returns the string representing the noise
     *
     * @return  string describing the noise
     */
    public String getNoise()
    {
        String selectedItem = (String) noiseComboBox.getSelectedItem();

        return selectedItem;
    }

    /**
     * Returns the index of the selected noise
     *
     * @return index of the selected noise
     */
    public int getNoiseIndex()
    {
        int selectedIndex = noiseComboBox.getSelectedIndex();

        return selectedIndex;
    }

    /**
     * Returns the degrees of freedom (used by certain noise distributions)
     *
     * @return integer in range 1..10
     */
    public int getDegreesFreedom()
    {
        return Integer.parseInt(degreesFreedomTextField.getText().trim());
    }

    /**
     * Returns the string representing the filename
     *
     * @return string describing the filename
     */
    public String getRawDataFileName()
    {
        return rawDataFileNameTextField.getText().trim();
    }

    /**
     * Returns boolean that tells whether data should be generated or not.
     *
     * @return true or false
     */
    public boolean isDemoEnabled()
    {
        return (tabbedPane.getSelectedIndex() == DEMO_PANE);
    }

    // =========================================================================
    // PACKAGE INTERFACE
    // =========================================================================

    /**
     * Adds an accessibility description to JComponent
     *
     * @param component    <code>JComponent</code> object
     * @param propertyKey  key to look up description in app properties
     */
    void addAccessibleDescription(JComponent component, String propertyKey)
    {
        try
        {
            String description = getBundleString(propertyKey);
            AccessibleContext ac = component.getAccessibleContext();
            ac.setAccessibleDescription(description);
        }
        catch (MissingResourceException e)
        {
            // Inability to add description is simply an annoyance
            System.err.println(e);
        }
    }

    /**
     * Adds tooltip text to JComponent
     *
     * @param component    <code>JComponent</code> object
     * @param propertyKey  key to look up tooltip in app properties
     */
    void addToolTipText(JComponent component, String propertyKey)
    {
        try
        {
            String toolTipText = getBundleString(propertyKey);
            component.setToolTipText(toolTipText);
        }
        catch (MissingResourceException e)
        {
            // Inability to add tooltip text is simply an annoyance
            System.err.println(e);
        }
    }

    /**
     * Displays error message to user
     *
     * @param message  error message to be displayed
     */
    void displayError(String message)
    {
        displayErrorDialog(contentPane, message);
    }

    /**
     * Displays an error message dialog
     *
     * @param parent   container object
     * @param message  error message to be displayed
     */
    void displayErrorDialog(Container parent, String message)
    {
        final String title = getBundleString(ERROR_DIALOG_TITLE);
        GuiUtil.displayErrorDialog(parent, title, message);
    }

    /**
     * Returns the application menu bar
     *
     * @return  reference to <code>JMenuBar</code> object
     */
    JMenuBar getMenuBar()
    {
        return menuBar;
    }

    /**
     * Reads a new configuration from disk
     *
     * @param configFile  pathname which contains settings
     */
    void loadConfiguration(File configFile)
    {
        Settings settings = new Settings();

        try
        {
            // :TODO: Write something to handle a failed read...
            settings.read(configFile);

            // Copy values to individual interface components
            loadGenericConfigSettings(settings);
            loadRawDataConfigSettings(settings);
            loadDemoConfigSettings(settings);

            if ((settings.isDemoMode() == true) &&
                (isDemoEnabled() == false))
            {
                selectPane(DEMO_PANE);
            }
            else if ((settings.isDemoMode() == false) &&
                     (isDemoEnabled() == true))
            {
                selectPane(RAWDATA_PANE);
            }
        }
        catch (FileNotFoundException fnfe)
        {
            String formatStr = getBundleString(ERR_LOAD_FILE);
            String fileName = fnfe.getMessage();

            if (fileName == null)
                fileName = getBundleString(NO_EXCEPTION_DETAIL_MESSAGE);

            Object[] args = { fileName };
            MessageFormat mf = new MessageFormat(formatStr);
            final String errorMessage = mf.format(args);
            displayErrorDialog(contentPane, errorMessage);
            fnfe.printStackTrace();
        }
        catch (FileVersionMismatchException fvme)
        {
            String errorMessage = getBundleString(ERR_FILE_VERSION_MISMATCH);
            displayErrorDialog(contentPane, errorMessage);
            fvme.printStackTrace();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Creates application menu bar
     *
     * @return instance of <code>JMenuBar</code> object
     */
    private JMenuBar createMenuBar()
    {
        menuBar = new JMenuBar();
        addAccessibleDescription(menuBar, MENUBAR_DESCRIPTION);

        menuBar.setBorder(new BevelBorder(BevelBorder.RAISED));
        menuBar.setBorderPainted(true);

        JMenu fileMenu = createFileMenu();
        JMenu helpMenu = createHelpMenu();

        menuBar.add(fileMenu);
        menuBar.add(helpMenu);
        // -- Should be as below but currently unimplemented in JDK
        // menuBar.setHelpMenu(helpMenu);

        return menuBar;
    }

    /**
     * Creates panel containing the client-specific portion of the interface
     *
     * @return instance of <code>JPanel</code> object
     */
    private JPanel createClientPanel()
    {
        // Create internal panel that displays entry fields(vertical layout)
        JPanel entryPanel = new JPanel();
        entryPanel.setLayout(new BoxLayout(entryPanel, BoxLayout.Y_AXIS));
        entryPanel.setAlignmentX(0.5f);
        entryPanel.add(createControlArea());
        entryPanel.add(createActionArea());

        // Create outer panel for display purposes (horizontal layout)
        // Glue allows resizing w/o stretching internal panel.
        JPanel clientPanel = new JPanel();
        clientPanel.setLayout(new BoxLayout(clientPanel, BoxLayout.X_AXIS));
        clientPanel.setBackground(GeneClustPanel.steelblue);
        clientPanel.add(Box.createHorizontalGlue());
        clientPanel.add(entryPanel);
        clientPanel.add(Box.createHorizontalGlue());

        return clientPanel;
    }

    /**
     * Creates application FILE menu and associated menu items
     *
     * @return instance of <code>JMenu</code> object
     */
    private JMenu createFileMenu()
    {
        String label;

        // Make sure this label is here, regardless
        try
        {
            label = getBundleString(FILE_MENU_LABEL);
        }
        catch (MissingResourceException e)
        {
            label = "File";
        }

        // Add the FiLE menu
        JMenu fileMenu = new JMenu(label, true);
        addAccessibleDescription(fileMenu, FILE_MENU_DESCRIPTION);

        String menuTextStr = fileMenu.getText();
        fileMenu.setMnemonic(menuTextStr.charAt(0));

        addFileMenuItems(fileMenu);

        return fileMenu;
    }

    /**
     * Adds associated menu items to the FILE menu
     *
     * @param fileMenu  object reference to the application's FILE menu
     */
    private void addFileMenuItems(JMenu fileMenu)
    {
        JMenuItem openMenuItem;
        JMenuItem saveMenuItem;
        JMenuItem exitMenuItem;
        KeyStroke keyStroke;

        // Open
        {
            final String[] keys =
            {
                OPEN_MENUITEM_LABEL,
                OPEN_MENUITEM_MNEMONIC,
                OPEN_MENUITEM_DESCRIPTION,
                OPEN_MENUITEM_TOOLTIP
            };

            openMenuItem = createMenuItem(keys,
                                   new OpenConfigFileSelectionButtonListener());
            keyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_O,
                                               InputEvent.CTRL_MASK);
            openMenuItem.setAccelerator(keyStroke);
        }

        // Save
        {
            final String[] keys =
            {
                SAVE_MENUITEM_LABEL,
                SAVE_MENUITEM_MNEMONIC,
                SAVE_MENUITEM_DESCRIPTION,
                SAVE_MENUITEM_TOOLTIP
            };

            saveMenuItem = createMenuItem(keys,
                                  new SaveConfigFileSelectionButtonListener());
            keyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_S,
                                               InputEvent.CTRL_MASK);
            saveMenuItem.setAccelerator(keyStroke);
        }

        // Exit
        {
            final String[] keys =
            {
                EXIT_MENUITEM_LABEL,
                EXIT_MENUITEM_MNEMONIC,
                EXIT_MENUITEM_DESCRIPTION,
                EXIT_MENUITEM_TOOLTIP
            };

            exitMenuItem = createMenuItem(keys,
                                      new ExitButtonListener());

            // Make sure this label is here, regardless
            try
            {
                String label = getBundleString(EXIT_MENUITEM_LABEL);
            }
            catch (MissingResourceException e)
            {
                final String HARDCODED_EXIT_LABEL = "Exit";
                exitMenuItem.setText(HARDCODED_EXIT_LABEL);
            }
        }

        fileMenu.add(openMenuItem);
        fileMenu.add(saveMenuItem);
        fileMenu.addSeparator();
        fileMenu.add(exitMenuItem);
    }

    /**
     * Creates application HELP menu and associated menu items
     *
     * @return instance of <code>JMenu</code> object
     */
    private JMenu createHelpMenu()
    {
        // Add the HELP menu
        final String label = getBundleString(HELP_MENU_LABEL);
        JMenu helpMenu = new JMenu(label, true);
        addAccessibleDescription(helpMenu, HELP_MENU_DESCRIPTION);

        String menuTextStr = helpMenu.getText();
        helpMenu.setMnemonic(menuTextStr.charAt(0));

        addHelpMenuItems(helpMenu);

        return helpMenu;
    }

    /**
     * Adds associated menu items to the HELP menu
     *
     * @param helpMenu  object reference to the application's HELP menu
     */
    private void addHelpMenuItems(JMenu helpMenu)
    {
        JMenuItem overviewMenuItem;
        JMenuItem userGuideMenuItem;
        JMenuItem aboutMenuItem;

        // Overview
        {
            final String[] keys =
            {
                OVERVIEW_MENUITEM_LABEL,
                OVERVIEW_MENUITEM_MNEMONIC,
                OVERVIEW_MENUITEM_DESCRIPTION,
                OVERVIEW_MENUITEM_TOOLTIP
            };

            overviewMenuItem = createMenuItem(keys,
                                          new OverviewButtonListener());
        }

        // User Guide
        {
            final String[] keys =
            {
                USER_GUIDE_MENUITEM_LABEL,
                USER_GUIDE_MENUITEM_MNEMONIC,
                USER_GUIDE_MENUITEM_DESCRIPTION,
                USER_GUIDE_MENUITEM_TOOLTIP
            };

            userGuideMenuItem = createMenuItem(keys,
                                           new UserGuideButtonListener());
        }

        // About
        {
            final String[] keys =
            {
                ABOUT_MENUITEM_LABEL,
                ABOUT_MENUITEM_MNEMONIC,
                ABOUT_MENUITEM_DESCRIPTION,
                ABOUT_MENUITEM_TOOLTIP
            };

            aboutMenuItem = createMenuItem(keys,
                                       new AboutButtonListener());
        }

        helpMenu.add(overviewMenuItem);
        helpMenu.add(userGuideMenuItem);
        helpMenu.addSeparator();
        helpMenu.add(aboutMenuItem);
    }

    /**
     * Creates panel containing the data entry portion of the interface
     *
     * @return instance of <code>JPanel</code> object
     */
    private JPanel createControlArea()
    {
        tabbedPane = new JTabbedPane();
        JPanel rawDataTab = createRawDataPanel();
        JPanel demoDataTab = createDemoDataPanel();
        String rawDataTabTitle = getBundleString(RAW_DATA_TAB_TITLE);
        String demoDataTabTitle = getBundleString(DEMO_DATA_TAB_TITLE);
        tabbedPane.add(rawDataTab, rawDataTabTitle);
        tabbedPane.add(demoDataTab, demoDataTabTitle);

        JPanel controlArea = new JPanel();

        final int NUM_ROWS = 2;
        controlArea.setLayout(new VerticalFlowLayout(NUM_ROWS));
        controlArea.setBackground(GeneClustPanel.steelblue);
        controlArea.add(tabbedPane);
        controlArea.add(createGenericDataEntryArea());

        return controlArea;
    }

    /**
     * Creates panel containing the user actions portion of the interface
     *
     * @return instance of <code>JPanel</code> object
     */
    private JPanel createActionArea()
    {
        // =====================================================================
        // INNER CLASSES
        // =====================================================================

        /**
         * MonitoringButton - Enables button when monitored fields are valid.
         *
         * @version 0.95 2000/12/27
         * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
         */
        class MonitoringButton extends JButton
          implements StatusListener
        {
            // =================================================================
            // CONSTRUCTORS
            // =================================================================

            /**
             * Class constructor specifying label text
             *
             * @param labelStr  string specifying button label text
             */
            public MonitoringButton(String labelStr)
            {
                super(labelStr);
            }

            // =================================================================
            // IMPLEMENTS STATUSLISTENER
            // =================================================================

            /**
             * Notifies observers when the state of the data changes
             * Enables button is data is valid; otherwise, disables it
             *
             * @param event  status event
             */
            public void statusChanged(StatusEvent event)
            {
// :BUG: This is broken. Works fine if only one item is invalid at a time but
//       incorrectly if two are invalid and one is corrected, the button will
//       be enabled although it should not be.
                this.setEnabled(event.getStatus());
            }
        }

        JPanel actionArea = new JPanel();
        actionArea.setBackground(GeneClustPanel.steelblue);

        Font sansSerif = new Font("SansSerif", Font.BOLD, 20);

	// Shave button (Development in progress...)
        MonitoringButton shaveButton = new MonitoringButton("Shave");
        shaveButton.setFont(sansSerif);
        shaveButton.setForeground(Color.white);
        shaveButton.setBackground(GeneClustPanel.firebrick);
        shaveButton.addActionListener(new DisplayButtonListener("shave"));
        actionArea.add(shaveButton);

	// Display button (default)
        String labelStr = getBundleString(DISPLAY_BUTTON_LABEL);
        String toolTip = getBundleString(DISPLAY_BUTTON_TOOLTIP);

        MonitoringButton displayButton = new MonitoringButton(labelStr);
        addAccessibleDescription(displayButton, DISPLAY_BUTTON_DESCRIPTION);

        displayButton.setToolTipText(toolTip);
        displayButton.setFont(sansSerif);
        displayButton.setForeground(Color.white);
        displayButton.setBackground(GeneClustPanel.firebrick);
        displayButton.addActionListener(new DisplayButtonListener("display"));
        actionArea.add(displayButton);

        // Set this button as the default button for the application
        displayButton.setDefaultCapable(true);

        // Disable this button if textfield is invalid
        addStatusListener(displayButton);

	// Print button (development in progress...)
        MonitoringButton printButton = new MonitoringButton("Print");
        printButton.setFont(sansSerif);
        printButton.setForeground(Color.white);
        printButton.setBackground(GeneClustPanel.firebrick);
        printButton.addActionListener(new DisplayButtonListener("print"));
        actionArea.add(printButton);

        return actionArea;
    }

    /**
     * Creates panel containing the generic data entry portion of the interface
     *
     * @return instance of <code>JPanel</code> object
     */
    private JPanel createGenericDataEntryArea()
    {
        JPanel genericDataPanel = new JPanel();


        genericDataPanel.setLayout(new VerticalFlowLayout());
        genericDataPanel.add(createShavingPanel());

        genericDataPanel.add(createHierarchicalClusteringPanel());

        return genericDataPanel;
    }

    /**
     * Creates panel containing the hierarchical clustering fields
     *
     * @return instance of <code>JPanel</code> object
     */
    private JPanel createHierarchicalClusteringPanel()
    {
        JPanel borderedPane;
        String labelStr;
        String description;
        String toolTip;
        String defaultValue;
        boolean selected;
        Font sansSerif = new Font("SansSerif", Font.BOLD, 14);

        borderedPane = new JPanel();
        borderedPane.setForeground(Color.white);
        borderedPane.setBackground(GeneClustPanel.steelblue);
        borderedPane.setLayout(new BoxLayout(borderedPane, BoxLayout.Y_AXIS));

        final int GAP = 15;
	final int IconTextGap = 8;

        borderedPane.add(Box.createRigidArea(new Dimension(GAP, 0)));

	// Panel border and title
        {
            TitledBorder titledBorder;
            String title;

            title = getBundleString(DISPLAY_PANEL_TITLE);
            titledBorder = BorderFactory.createTitledBorder(title);
            titledBorder.setTitleColor(Color.white);
            borderedPane.setBorder(titledBorder);
        }

        // Display Data Matrix
        {
            labelStr = getBundleString(DISPLAY_DATA_MATRIX_CHECKBOX_LABEL);
            description =
                  getBundleString(DISPLAY_DATA_MATRIX_CHECKBOX_DESCRIPTION);
            toolTip = getBundleString(DISPLAY_DATA_MATRIX_CHECKBOX_TOOLTIP);
            defaultValue =
                  getBundleString(DISPLAY_DATA_MATRIX_CHECKBOX_DEFAULT_VALUE);
            selected = new Boolean(defaultValue).booleanValue();

            displayDataMatrixCheckBox = new JCheckBox(labelStr, selected);

            // float indent = 0.13f;
            // displayDataMatrixCheckBox.setAlignmentX(indent);
            displayDataMatrixCheckBox.setIconTextGap(IconTextGap);
            displayDataMatrixCheckBox.setBackground(GeneClustPanel.steelblue);
            displayDataMatrixCheckBox.setFocusPainted(true);
            displayDataMatrixCheckBox.setFont(sansSerif);
            displayDataMatrixCheckBox.setForeground(Color.yellow);
            AccessibleContext ac =
                       displayDataMatrixCheckBox.getAccessibleContext();
            ac.setAccessibleDescription(description);
            displayDataMatrixCheckBox.setToolTipText(description);
            borderedPane.add(displayDataMatrixCheckBox);
        }

        // Display Hierarchial Clusters by Genes
        {
            labelStr = getBundleString(DISPLAY_GENES_GRAPH_CHECKBOX_LABEL);
            description =
                      getBundleString(DISPLAY_GENES_GRAPH_CHECKBOX_DESCRIPTION);
            toolTip = getBundleString(DISPLAY_GENES_GRAPH_CHECKBOX_TOOLTIP);
            defaultValue =
                   getBundleString(DISPLAY_GENES_GRAPH_CHECKBOX_DEFAULT_VALUE);
            selected = new Boolean(defaultValue).booleanValue();

            displayGenesGraphCheckBox = new JCheckBox(labelStr, selected);

            // float indent = 0.13f;
            // displayGenesGraphCheckBox.setAlignmentX(indent);
            displayGenesGraphCheckBox.setIconTextGap(IconTextGap);
            displayGenesGraphCheckBox.setBackground(GeneClustPanel.steelblue);
            displayGenesGraphCheckBox.setFocusPainted(false);
            displayGenesGraphCheckBox.setFont(sansSerif);
            displayGenesGraphCheckBox.setForeground(Color.yellow);
            AccessibleContext ac =
                         displayGenesGraphCheckBox.getAccessibleContext();
            ac.setAccessibleDescription(description);
            displayGenesGraphCheckBox.setToolTipText(description);
            borderedPane.add(displayGenesGraphCheckBox);
        }

        // Display Hierarchial Clusters by Samples
        {
            labelStr = getBundleString(DISPLAY_SAMPLES_GRAPH_CHECKBOX_LABEL);
            description =
                  getBundleString(DISPLAY_SAMPLES_GRAPH_CHECKBOX_DESCRIPTION);
            toolTip = getBundleString(DISPLAY_SAMPLES_GRAPH_CHECKBOX_TOOLTIP);
            defaultValue =
                  getBundleString(DISPLAY_SAMPLES_GRAPH_CHECKBOX_DEFAULT_VALUE);
            selected = new Boolean(defaultValue).booleanValue();

            displaySamplesGraphCheckBox = new JCheckBox(labelStr, selected);

            // float indent = 0.13f;
            // displaySamplesGraphCheckBox.setAlignmentX(indent);
            displaySamplesGraphCheckBox.setIconTextGap(IconTextGap);
            displaySamplesGraphCheckBox.setBackground(GeneClustPanel.steelblue);
            displaySamplesGraphCheckBox.setFocusPainted(false);
            displaySamplesGraphCheckBox.setFont(sansSerif);
            displaySamplesGraphCheckBox.setForeground(Color.yellow);
            AccessibleContext ac =
                       displaySamplesGraphCheckBox.getAccessibleContext();
            ac.setAccessibleDescription(description);
            displaySamplesGraphCheckBox.setToolTipText(description);
            borderedPane.add(displaySamplesGraphCheckBox);
        }

        // Display Gene Shaving Clusters
        {
            labelStr = getBundleString(DISPLAY_SHAVING_CLUSTERS_CHECKBOX_LABEL);
            description =
                  getBundleString(DISPLAY_SHAVING_CLUSTERS_CHECKBOX_DESCRIPTION);
            toolTip = getBundleString(DISPLAY_SHAVING_CLUSTERS_CHECKBOX_TOOLTIP);
            defaultValue =
                  getBundleString(DISPLAY_SHAVING_CLUSTERS_CHECKBOX_DEFAULT_VALUE);
            selected = new Boolean(defaultValue).booleanValue();

            displayShavingCheckBox = new JCheckBox(labelStr, selected);

            // float indent = 0.13f;
            // displayShavingCheckBox.setAlignmentX(indent);
            displayShavingCheckBox.setIconTextGap(IconTextGap);
            displayShavingCheckBox.setBackground(GeneClustPanel.steelblue);
            displayShavingCheckBox.setFocusPainted(true);
            displayShavingCheckBox.setFont(sansSerif);
            displayShavingCheckBox.setForeground(Color.yellow);
            AccessibleContext ac =
                       displayShavingCheckBox.getAccessibleContext();
            ac.setAccessibleDescription(description);
            displayShavingCheckBox.setToolTipText(description);
            borderedPane.add(displayShavingCheckBox);
        }

        return borderedPane;
    }

    /**
     * Creates panel containing the supervised shaving fields
     *
     * @return instance of <code>JPanel</code> object
     */
    private JPanel createShavingPanel()
    {
        JPanel borderedPane;
        String propertyKey;
        String labelStr;
        String description;
        String toolTip;
        String defaultValue;
        boolean selected;
        Font sansSerif = new Font("SansSerif", Font.BOLD, 14);

        borderedPane = new JPanel();
        borderedPane.setForeground(Color.white);
        borderedPane.setBackground(GeneClustPanel.steelblue);
        borderedPane.setLayout(new BoxLayout(borderedPane, BoxLayout.Y_AXIS));

        {
            TitledBorder titledBorder;
            String title;

            title = getBundleString(SHAVING_PANEL_TITLE);
            titledBorder = BorderFactory.createTitledBorder(title);
            titledBorder.setTitleColor(Color.white);
            borderedPane.setBorder(titledBorder);
        }

        final int NUM_ROWS = UNLIMITED;
        final int NUM_COLS = 2;
        final int GAP = 1;

        JPanel inner = new JPanel();
        inner.setLayout(new GridLayout(NUM_ROWS, NUM_COLS, GAP, GAP));
        inner.setBackground(GeneClustPanel.steelblue);

        // Number of Clusters to Extract
        {
            final String[] keys =
            {
                NUM_CLUSTERS_TO_EXTRACT_LABEL,
                NUM_CLUSTERS_TO_EXTRACT_TEXTFIELD_COLUMNS,
                NUM_CLUSTERS_TO_EXTRACT_TEXTFIELD_DESCRIPTION,
                NUM_CLUSTERS_TO_EXTRACT_TEXTFIELD_TOOLTIP,
                NUM_CLUSTERS_TO_EXTRACT_TEXTFIELD_DEFAULT_VALUE,
                NUM_CLUSTERS_TO_EXTRACT_TEXTFIELD_MINIMUM_VALUE,
                NUM_CLUSTERS_TO_EXTRACT_TEXTFIELD_MAXIMUM_VALUE
            };

            numExtClustersTextField = createLabelTextField(keys, inner);
        }

        // Percent to Shave
        {
            final String[] keys =
            {
                PERCENT_TO_SHAVE_LABEL,
                PERCENT_TO_SHAVE_TEXTFIELD_COLUMNS,
                PERCENT_TO_SHAVE_TEXTFIELD_DESCRIPTION,
                PERCENT_TO_SHAVE_TEXTFIELD_TOOLTIP,
                PERCENT_TO_SHAVE_TEXTFIELD_DEFAULT_VALUE,
                PERCENT_TO_SHAVE_TEXTFIELD_MINIMUM_VALUE,
                PERCENT_TO_SHAVE_TEXTFIELD_MAXIMUM_VALUE
            };

            percentToShaveTextField = createLabelTextField(keys, inner);
        }

        // Number of Permutations
        {
            final String[] keys =
            {
                NUM_PERMUTATIONS_LABEL,
                NUM_PERMUTATIONS_TEXTFIELD_COLUMNS,
                NUM_PERMUTATIONS_TEXTFIELD_DESCRIPTION,
                NUM_PERMUTATIONS_TEXTFIELD_TOOLTIP,
                NUM_PERMUTATIONS_TEXTFIELD_DEFAULT_VALUE,
                NUM_PERMUTATIONS_TEXTFIELD_MINIMUM_VALUE,
                NUM_PERMUTATIONS_TEXTFIELD_MAXIMUM_VALUE
            };

            numPermutationsTextField = createLabelTextField(keys, inner);
        }

        // Gap Tolerance
        {
            final String[] keys =
            {
                GAP_TOLERANCE_LABEL,
                GAP_TOLERANCE_TEXTFIELD_COLUMNS,
                GAP_TOLERANCE_TEXTFIELD_DESCRIPTION,
                GAP_TOLERANCE_TEXTFIELD_TOOLTIP,
                GAP_TOLERANCE_TEXTFIELD_DEFAULT_VALUE,
                GAP_TOLERANCE_TEXTFIELD_MINIMUM_VALUE,
                GAP_TOLERANCE_TEXTFIELD_MAXIMUM_VALUE
            };

            gapToleranceTextField = createLabelTextField(keys, inner);
        }
        borderedPane.add(inner);

        // Perform Supervised
        if (false) {
            JPanel panel = new JPanel();
            panel.setBackground(GeneClustPanel.steelblue);
            panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

            // final int GAP = 15;
            panel.add(Box.createRigidArea(new Dimension(GAP, 0)));

            propertyKey = PERFORM_SUPERVISED_CHECKBOX_LABEL;
            labelStr = getBundleString(propertyKey);
            propertyKey = PERFORM_SUPERVISED_CHECKBOX_DESCRIPTION;
            description = getBundleString(propertyKey);
            propertyKey = PERFORM_SUPERVISED_CHECKBOX_TOOLTIP;
            toolTip = getBundleString(propertyKey);
            propertyKey = PERFORM_SUPERVISED_CHECKBOX_DEFAULT_VALUE;
            defaultValue = getBundleString(propertyKey);
            selected = new Boolean(defaultValue).booleanValue();

            performSupervisedCheckBox = new JCheckBox(labelStr, selected);

            float indent = 0.16f;
            performSupervisedCheckBox.setAlignmentX(indent);
            performSupervisedCheckBox.setBackground(GeneClustPanel.steelblue);
            performSupervisedCheckBox.setFocusPainted(false);
            performSupervisedCheckBox.setFont(sansSerif);
            performSupervisedCheckBox.setForeground(Color.yellow);

            performSupervisedCheckBox.addActionListener(
                                   new PerformSupervisedCheckBoxListener());
            AccessibleContext ac =
                             performSupervisedCheckBox.getAccessibleContext();
            ac.setAccessibleDescription(description);
            performSupervisedCheckBox.setToolTipText(description);
            panel.add(performSupervisedCheckBox);
            panel.add(Box.createHorizontalGlue());

            borderedPane.add(panel);
        }
	else
	    selected = true;

        // Percent Supervision
        {
            JPanel panel = new JPanel();

            // final int NUM_ROWS = UNLIMITED;
            // final int NUM_COLS = 2;
            // final int GAP = 1;

            panel.setLayout(new GridLayout(NUM_ROWS, NUM_COLS, GAP, GAP));
            panel.setBackground(GeneClustPanel.steelblue);

            {
                final String[] keys =
                {
                    PERCENT_SUPERVISION_LABEL,
                    PERCENT_SUPERVISION_TEXTFIELD_COLUMNS,
                    PERCENT_SUPERVISION_TEXTFIELD_DESCRIPTION,
                    PERCENT_SUPERVISION_TEXTFIELD_TOOLTIP,
                    PERCENT_SUPERVISION_TEXTFIELD_DEFAULT_VALUE,
                    PERCENT_SUPERVISION_TEXTFIELD_MINIMUM_VALUE,
                    PERCENT_SUPERVISION_TEXTFIELD_MAXIMUM_VALUE
                };

                percentSupervisionTextField = createLabelTextField(keys, panel);
                percentSupervisionTextField.setEnabled(selected);
            }

            borderedPane.add(panel);
        }

        // Classification Filename
        {
            AccessibleContext ac;
            JPanel panel = new JPanel();
            panel.setBackground(GeneClustPanel.steelblue);

            Font monospaced = new Font("Monospaced", Font.PLAIN, 14);
            propertyKey = CLASSIFICATION_FILENAME_TEXTFIELD_DESCRIPTION;
            description = getBundleString(propertyKey);
            propertyKey = CLASSIFICATION_FILENAME_TEXTFIELD_TOOLTIP;
            toolTip = getBundleString(propertyKey);
            propertyKey = CLASSIFICATION_FILENAME_TEXTFIELD_DEFAULT_VALUE;
            defaultValue = getBundleString(propertyKey);
            propertyKey = CLASSIFICATION_FILENAME_TEXTFIELD_COLUMNS;
            int columns = Integer.parseInt(getBundleString(propertyKey));
            PathnameTextField textField;
            textField = new PathnameTextField(defaultValue, columns);
            textField.setFont(monospaced);
            textField.setEnabled(selected);
            ac = textField.getAccessibleContext();
            ac.setAccessibleDescription(description);
            textField.setToolTipText(toolTip);
            textField.addFocusListener(new AutoSelectEventHandler());

            labelStr = getBundleString(CLASSIFICATION_FILENAME_LABEL);
            JLabel label = new JLabel(labelStr, JLabel.RIGHT);
            label.setFont(sansSerif);
            label.setForeground(Color.yellow);
            label.setLabelFor(textField);

            propertyKey = CLASSIFICATION_FILENAME_CHOOSER_BUTTON_LABEL;
            String buttonLabelStr = getBundleString(propertyKey);
            propertyKey = CLASSIFICATION_FILENAME_CHOOSER_BUTTON_DESCRIPTION;
            description = getBundleString(propertyKey);
            propertyKey = CLASSIFICATION_FILENAME_CHOOSER_BUTTON_TOOLTIP;
            toolTip = getBundleString(propertyKey);
            classificationFileChooserButton = new JButton(buttonLabelStr);
            ac = classificationFileChooserButton.getAccessibleContext();
            ac.setAccessibleDescription(description);
            classificationFileChooserButton.setToolTipText(toolTip);

            classificationFileChooserButton.addActionListener(
                       new ChooseClassificationFileSelectionButtonListener());
            classificationFileChooserButton.setEnabled(selected);

            panel.add(label);
            panel.add(textField);
            panel.add(classificationFileChooserButton);

            classificationFileNameTextField = textField;

            borderedPane.add(panel);
        }

        return borderedPane;
    }

    /**
     * Creates panel containing the rawdata data entry portion of the interface
     *
     * @return instance of <code>JPanel</code> object
     */
    private JPanel createRawDataPanel()
    {
        JPanel rawDataPanel = new JPanel();
        rawDataPanel.setBackground(GeneClustPanel.steelblue);

        JPanel panel = new JPanel();
        panel.setBackground(GeneClustPanel.steelblue);

        String propertyKey;
        AccessibleContext ac;
        Font sansSerif = new Font("SansSerif", Font.BOLD, 14);
        Font monospaced = new Font("Monospaced", Font.PLAIN, 14);

        propertyKey = RAW_DATA_FILENAME_TEXTFIELD_DESCRIPTION;
        String description = getBundleString(propertyKey);
        propertyKey = RAW_DATA_FILENAME_TEXTFIELD_TOOLTIP;
        String toolTip = getBundleString(propertyKey);
        propertyKey = RAW_DATA_FILENAME_TEXTFIELD_DEFAULT_VALUE;
        String defaultValue = getBundleString(propertyKey);
        propertyKey = RAW_DATA_FILENAME_TEXTFIELD_COLUMNS;
        int columns = Integer.parseInt(getBundleString(propertyKey));
        PathnameTextField textField;
        textField = new PathnameTextField(defaultValue, columns);
        textField.setFont(monospaced);
        ac = textField.getAccessibleContext();
        ac.setAccessibleDescription(description);
        textField.setToolTipText(toolTip);
        textField.addFocusListener(new AutoSelectEventHandler());

        String labelStr = getBundleString(RAW_DATA_FILENAME_LABEL);
        JLabel label = new JLabel(labelStr, JLabel.RIGHT);
        label.setFont(sansSerif);
        label.setLabelFor(textField);
        label.setForeground(Color.yellow);

        propertyKey = RAW_DATA_FILENAME_CHOOSER_BUTTON_LABEL;
        String buttonLabelStr = getBundleString(propertyKey);
        propertyKey = RAW_DATA_FILENAME_CHOOSER_BUTTON_DESCRIPTION;
        description = getBundleString(propertyKey);
        propertyKey = RAW_DATA_FILENAME_CHOOSER_BUTTON_TOOLTIP;
        toolTip = getBundleString(propertyKey);
        JButton rawDataFileChooserButton = new JButton(buttonLabelStr);
        ac = rawDataFileChooserButton.getAccessibleContext();
        ac.setAccessibleDescription(description);
        rawDataFileChooserButton.setToolTipText(toolTip);
        rawDataFileChooserButton.addActionListener(
                                 new ChooseDataFileSelectionButtonListener());

        panel.add(label);
        panel.add(textField);
        panel.add(rawDataFileChooserButton);

        rawDataFileNameTextField = textField;

        rawDataPanel.add(panel);

        return rawDataPanel;
    }

    /**
     * Creates panel containing the demo data entry portion of the interface
     *
     * @return instance of <code>JPanel</code> object
     */
    private JPanel createDemoDataPanel()
    {
        JPanel demoPanel = new JPanel();

        final int NUM_ROWS = UNLIMITED;
        final int NUM_COLS = 2;
        final int GAP = 1;

        demoPanel.setLayout(new GridLayout(NUM_ROWS, NUM_COLS, GAP, GAP));
        demoPanel.setBackground(GeneClustPanel.steelblue);
        demoPanel.setBorder(new BevelBorder(BevelBorder.LOWERED));

        // Number of Rows
        {
            final String[] keys =
            {
                NUM_ROWS_LABEL,
                NUM_ROWS_TEXTFIELD_COLUMNS,
                NUM_ROWS_TEXTFIELD_DESCRIPTION,
                NUM_ROWS_TEXTFIELD_TOOLTIP,
                NUM_ROWS_TEXTFIELD_DEFAULT_VALUE,
                NUM_ROWS_TEXTFIELD_MINIMUM_VALUE,
                NUM_ROWS_TEXTFIELD_MAXIMUM_VALUE
            };

            numRowsTextField = createLabelTextField(keys, demoPanel);
        }

        // Number of Columns
        {
            final String[] keys =
            {
                NUM_COLUMNS_LABEL,
                NUM_COLUMNS_TEXTFIELD_COLUMNS,
                NUM_COLUMNS_TEXTFIELD_DESCRIPTION,
                NUM_COLUMNS_TEXTFIELD_TOOLTIP,
                NUM_COLUMNS_TEXTFIELD_DEFAULT_VALUE,
                NUM_COLUMNS_TEXTFIELD_MINIMUM_VALUE,
                NUM_COLUMNS_TEXTFIELD_MAXIMUM_VALUE
            };

            numColumnsTextField = createLabelTextField(keys, demoPanel);
        }

        // Amplitude
        {
            final String[] keys =
            {
                AMPLITUDE_LABEL,
                AMPLITUDE_TEXTFIELD_COLUMNS,
                AMPLITUDE_TEXTFIELD_DESCRIPTION,
                AMPLITUDE_TEXTFIELD_TOOLTIP,
                AMPLITUDE_TEXTFIELD_DEFAULT_VALUE,
                AMPLITUDE_TEXTFIELD_MINIMUM_VALUE,
                AMPLITUDE_TEXTFIELD_MAXIMUM_VALUE
            };

            amplitudeTextField = createLabelTextField(keys, demoPanel);
        }

        // Model
        {
            String labelStr = getBundleString(MODEL_LABEL);
            String toolTip = getBundleString(MODEL_COMBOBOX_TOOLTIP);
            Font sansSerif = new Font("SansSerif", Font.BOLD, 14);

            JLabel label = new JLabel(labelStr, JLabel.RIGHT);
            label.setFont(sansSerif);
            label.setForeground(Color.yellow);
            label.setLabelFor(modelComboBox);
            demoPanel.add(label);

            Object[] comboBoxItems = getModelItems();
            modelComboBox = new JComboBox(comboBoxItems);
            addAccessibleDescription(modelComboBox, MODEL_COMBOBOX_DESCRIPTION);
            modelComboBox.setBackground(Color.white);
            modelComboBox.setToolTipText(toolTip);

            demoPanel.add(modelComboBox);
        }

        // Noise
        {
            String labelStr = getBundleString(NOISE_LABEL);
            String toolTip = getBundleString(NOISE_COMBOBOX_TOOLTIP);
            Font sansSerif = new Font("SansSerif", Font.BOLD, 14);

            JLabel label = new JLabel(labelStr, JLabel.RIGHT);
            label.setFont(sansSerif);
            label.setForeground(Color.yellow);
            label.setLabelFor(noiseComboBox);

            demoPanel.add(label);

            Object[] comboBoxItems = getNoiseItems();
            noiseComboBox = new JComboBox(comboBoxItems);
            addAccessibleDescription(noiseComboBox, NOISE_COMBOBOX_DESCRIPTION);
            noiseComboBox.setSelectedIndex(1);
            noiseComboBox.setBackground(Color.white);
            noiseComboBox.setToolTipText(toolTip);

            demoPanel.add(noiseComboBox);
        }

        // Degrees of Freedom
        {
            final String[] keys =
            {
                DEGREES_FREEDOM_LABEL,
                DEGREES_FREEDOM_TEXTFIELD_COLUMNS,
                DEGREES_FREEDOM_TEXTFIELD_DESCRIPTION,
                DEGREES_FREEDOM_TEXTFIELD_TOOLTIP,
                DEGREES_FREEDOM_TEXTFIELD_DEFAULT_VALUE,
                DEGREES_FREEDOM_TEXTFIELD_MINIMUM_VALUE,
                DEGREES_FREEDOM_TEXTFIELD_MAXIMUM_VALUE
            };

            degreesFreedomTextField = createLabelTextField(keys, demoPanel);
        }

        return demoPanel;
    }

    /**
     * Add status listener to input fields. If invalid entry is made,
     * the button will be set insensitive (disabled).
     *
     * @param listener  field to enable/disable depending on input
     */
    private void addStatusListener(StatusListener listener)
    {
        percentSupervisionTextField.addStatusListener(listener);
        classificationFileNameTextField.addStatusListener(listener);
        numExtClustersTextField.addStatusListener(listener);
        percentToShaveTextField.addStatusListener(listener);
        numPermutationsTextField.addStatusListener(listener);
        gapToleranceTextField.addStatusListener(listener);
        rawDataFileNameTextField.addStatusListener(listener);
        numRowsTextField.addStatusListener(listener);
        numColumnsTextField.addStatusListener(listener);
        amplitudeTextField.addStatusListener(listener);
        degreesFreedomTextField.addStatusListener(listener);
    }

    /**
     * Returns array of data models from application properties
     *
     * @return string array representing data models
     */
    private Object[] getModelItems()
    {
        final String basePropertyKey = "PK_MODEL_COMBOBOX_ITEM_LABEL_";

// :TODO: i18n
        return getComboBoxItems(basePropertyKey, "model");
    }

    /**
     * Returns array of noise types from application properties
     *
     * @return string array representing noise types
     */
    private Object[] getNoiseItems()
    {
        final String basePropertyKey = "PK_NOISE_COMBOBOX_ITEM_LABEL_";

// :TODO: i18n
        return getComboBoxItems(basePropertyKey, "noise");
    }

    /**
     * Returns array of combobox items from application properties
     *
     * @param basePropertyKey  string containing base property key
     * @param itemType         string representing type of item being retrieved.
     *                         Used in error message if something goes wrong.
     * @return                 string array representing data models
     */
    private Object[] getComboBoxItems(String basePropertyKey, String itemType)
    {
        List vector = new ArrayList();

        {
            String propertyKey;
            int itemNumber = 1;
            boolean done = false;

            while (done == false)
            {
                propertyKey = basePropertyKey + itemNumber;

                try
                {
                    String listItem = getBundleString(propertyKey);
                    vector.add(listItem.trim());
                    itemNumber++;
                }
                catch (MissingResourceException e)
                {
                    done = true;
                }
            }
        }

        // Copy vector contents to array, if possible
        String[] array = null;
        if (vector.size() != 0)
            array = (String[]) vector.toArray(new String[0]);
        else
        {
            String formatStr = getBundleString(ERR_NO_PROPERTY_ITEMS_FOUND);
            Object[] args = { itemType };
            MessageFormat mf = new MessageFormat(formatStr);
            final String errorMessage = mf.format(args);

            System.err.println(errorMessage);
            invokeTerminate(-1);
        }

        return array;
    }

    /**
     * Creates label/textfield combo and adds it to supplied parent container
     *
     * @param keys    resource bundle keys for combo creation
     * @param parent  container object for textfield/label combo
     * @return        instance of <code>NumericTextField</code> object
     */
    private NumericTextField createLabelTextField(final String[] keys,
                                                  JPanel parent)
    {
        int i = 0;

        String labelStr     = getBundleString(keys[i++]);
        String columns      = getBundleString(keys[i++]);
        String description  = getBundleString(keys[i++]);
        String toolTip      = getBundleString(keys[i++]);
        String defaultValue = getBundleString(keys[i++]);
        String minimumValue = getBundleString(keys[i++]);
        String maximumValue = getBundleString(keys[i++]);

        return createLabelTextField(labelStr,
                                    Integer.parseInt(columns),
                                    description,
                                    toolTip,
                                    defaultValue,
                                    minimumValue,
                                    maximumValue,
                                    parent);
    }

    /**
     * Creates label/textfield combo and adds it to supplied parent container
     *
     * @param labelStr      label displayed to the left of the textfield
     * @param columns       number of columns needed to display maximum value
     * @param description   accessibility description of the textfield
     * @param toolTip       tooltip displayed to the right of the textfield
     * @param defaultValue  default value of the textfield
     * @param minimumValue  minimum valid value
     * @param maximumValue  maximum valid value
     * @param parent        container object for textfield/label combo
     * @return              instance of <code>NumericTextField</code> object
     */
    private NumericTextField createLabelTextField(final String labelStr,
                                                  int columns,
                                                  final String description,
                                                  final String toolTip,
                                                  final String defaultValue,
                                                  final String minimumValue,
                                                  final String maximumValue,
                                                  JPanel parent)
    {
        Font sansSerif = new Font("SansSerif", Font.BOLD, 14);
        Font monospaced = new Font("Monospaced", Font.PLAIN, 14);
        NumericTextField textField;

        textField = new NumericTextField(defaultValue, columns);
        textField.setHorizontalAlignment(JTextField.RIGHT);
        textField.setFont(monospaced);

        // Set ranges as appropriate
        {
            double minimum = Double.parseDouble(minimumValue);
            double maximum = Double.parseDouble(maximumValue);

            textField.setRange(minimum, maximum);
        }

        AccessibleContext ac = textField.getAccessibleContext();
        ac.setAccessibleDescription(description);

        // Add ranges to existing tooltip
        {
            StringBuffer enhancedToolTip = new StringBuffer(64);

            enhancedToolTip.append(toolTip);
            enhancedToolTip.append("  [");
            enhancedToolTip.append(minimumValue);
            enhancedToolTip.append("..");
            enhancedToolTip.append(maximumValue);
            enhancedToolTip.append("]");

            textField.setToolTipText(enhancedToolTip.toString());
        }
        textField.addFocusListener(new AutoSelectEventHandler());

        JLabel label = new JLabel(labelStr, JLabel.RIGHT);
        label.setFont(sansSerif);
        label.setForeground(Color.yellow);
        label.setLabelFor(textField);

        parent.add(label);
        parent.add(textField);

        return textField;
    }

    /**
     * Displays an ABOUT informational dialog
     *
     * @param parent  container object
     */
    private void displayAboutDialog(Container parent)
    {
        if (aboutDialog == null)
        {
            String title = getBundleString(ABOUT_DIALOG_TITLE);
            aboutDialog = new GCAboutDialog(frame, title);

            // Set preferred dimensions
            {
                int WIDTH  = (int) aboutDialog.getPreferredSize().getWidth();
                int HEIGHT = (int) aboutDialog.getPreferredSize().getHeight();

                Insets insets = aboutDialog.getInsets();
                int width  = WIDTH  + (insets.left + insets.right);
                int height = HEIGHT + (insets.top + insets.bottom);
                aboutDialog.setSize(width, height);
            }
        }

        aboutDialog.display();
    }

    /**
     * Displays a FILE CHOOSER dialog for classification file selection
     *
     * @param parent  container object
     */
    private void displayChooseClassificationFileSelectionDialog(Container parent)
    {
        String propertyKey;

        // Create a file chooser
        propertyKey = CHOOSE_CLASSIFICATION_FILE_SELECTION_DIALOG_TITLE;
        String title = getBundleString(propertyKey);
        propertyKey = CHOOSE_FILE_SELECTION_DIALOG_OK_BUTTON_LABEL;
        String okButtonStr = getBundleString(propertyKey);
        propertyKey = CHOOSE_FILE_SELECTION_DIALOG_OK_BUTTON_TOOLTIP;
        String toolTip = getBundleString(propertyKey);

        File dataDirectory = new File(dataDirName);
        JFileChooser fileChooser = new JFileChooser(dataDirectory);
        GCFileFilter classificationFileFilter = new ClassificationFileFilter();
        fileChooser.addChoosableFileFilter(classificationFileFilter);
        fileChooser.setDialogTitle(title);
        fileChooser.setApproveButtonToolTipText(toolTip);

        int retCode = fileChooser.showDialog(parent, okButtonStr);

        if (retCode == JFileChooser.APPROVE_OPTION)
        {
            File file = fileChooser.getSelectedFile();
            classificationFileNameTextField.setText(file.getAbsolutePath());
        }
    }

    /**
     * Displays a FILE CHOOSER dialog for data file selection
     *
     * @param parent  container object
     */
    private void displayChooseDataFileSelectionDialog(Container parent)
    {
        String propertyKey;

        // Create a file chooser
        propertyKey = CHOOSE_DATA_FILE_SELECTION_DIALOG_TITLE;
        String title = getBundleString(propertyKey);
        propertyKey = CHOOSE_FILE_SELECTION_DIALOG_OK_BUTTON_LABEL;
        String okButtonStr = getBundleString(propertyKey);
        propertyKey = CHOOSE_FILE_SELECTION_DIALOG_OK_BUTTON_TOOLTIP;
        String toolTip = getBundleString(propertyKey);

        File dataDirectory = new File(dataDirName);
        JFileChooser fileChooser = new JFileChooser(dataDirectory);
        GCFileFilter defaultFileFilter = new TabSeparatedValuesFileFilter();
        fileChooser.addChoosableFileFilter(defaultFileFilter);
        fileChooser.setFileFilter(defaultFileFilter);
        fileChooser.setDialogTitle(title);
        fileChooser.setApproveButtonToolTipText(toolTip);

        int retCode = fileChooser.showDialog(parent, okButtonStr);

        if (retCode == JFileChooser.APPROVE_OPTION)
        {
            File file = fileChooser.getSelectedFile();
            rawDataFileNameTextField.setText(file.getAbsolutePath());
        }
    }

    /**
     * Displays a FILE CHOOSER dialog for loading configuration files
     *
     * @param parent  container object
     */
    private void displayOpenConfigFileSelectionDialog(Container parent)
    {
        String propertyKey = OPEN_SETTINGS_FILE_SELECTION_DIALOG_TITLE;
        String title = getBundleString(propertyKey);

        String configPathName = "conf";
        File configDirectory = new File(configPathName);
        JFileChooser fileChooser = new JFileChooser(configDirectory);
        fileChooser.addChoosableFileFilter(new ConfigFileFilter());
        fileChooser.setDialogTitle(title);

        int retCode = fileChooser.showOpenDialog(parent);
        if (retCode == JFileChooser.APPROVE_OPTION)
        {
            File file = fileChooser.getSelectedFile();
            loadConfiguration(file);
        }
    }

    /**
     * Displays an OVERVIEW informational dialog
     *
     * @param parent  container object
     */
    private void displayOverviewDialog(Container parent)
    {
        final String title = getBundleString(OVERVIEW_DIALOG_TITLE);
        final String message = getBundleString(OVERVIEW_DIALOG_MESSAGE);

        JOptionPane.showMessageDialog(parent,
                                      message,
                                      title,
                                      JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Displays a FILE CHOOSER dialog for saving configuration files
     *
     * @param parent  container object
     */
    private void displaySaveConfigFileSelectionDialog(Container parent)
    {
        String propertyKey = SAVE_SETTINGS_FILE_SELECTION_DIALOG_TITLE;
        String title = getBundleString(propertyKey);

        String configPathName = "conf";
        File configDirectory = new File(configPathName);
        JFileChooser fileChooser = new JFileChooser(configDirectory);
        fileChooser.addChoosableFileFilter(new ConfigFileFilter());
        fileChooser.setDialogTitle(title);

        int retCode = fileChooser.showSaveDialog(parent);
        if (retCode == JFileChooser.APPROVE_OPTION)
        {
            File file = fileChooser.getSelectedFile();

            // Add appropriate file extension, if necessary
            {
                final String configFileExtension = "cf";
                String extension = getExtension(file);

                if ((extension == null) ||
                    (extension.equals(configFileExtension) == false))
                {
                    String nameWithExtension = file.getAbsolutePath() +
                                               "." +
                                               configFileExtension;
                    file = new File(nameWithExtension);
                }
            }

            try
            {
                if (file.createNewFile() == false)
                {
                    title = getBundleString(OVERWRITE_DIALOG_TITLE);
                    propertyKey = OVERWRITE_DIALOG_MESSAGE;
                    String message = getBundleString(propertyKey);
                    int response = JOptionPane.showConfirmDialog(parent,
                                                                 message,
                                                                 title,
                                                     JOptionPane.YES_NO_OPTION);
                    if (response != JOptionPane.YES_OPTION)
                        return;
// :NOTE: Should file chooser be redisplayed instead of returning?
                }
            }
            catch (Exception e)
            {
                title = getBundleString(ERROR_DIALOG_TITLE);

                e.printStackTrace();
                JOptionPane.showMessageDialog(parent,
                                              e.getLocalizedMessage(),
                                              title,
                                              JOptionPane.ERROR_MESSAGE);
            }

            saveConfiguration(file);
        }
    }

    /**
     * Saves the current configuration to disk
     *
     * @param configFile  pathname to which to save settings
     */
    private void saveConfiguration(File configFile)
    {
        Settings settings = new Settings(isDemoEnabled());

        // Copy values from individual interface components to disk
        saveGenericConfigSettings(settings);
        saveRawDataConfigSettings(settings);
        saveDemoConfigSettings(settings);

        settings.write(configFile);
    }

    /**
     * Loads generic portion of configuration
     *
     * @param s  object reference containing values to load
     */
    private void loadGenericConfigSettings(Settings s)
    {
        numExtClustersTextField.setText(s.getNumExtClusters());
        percentToShaveTextField.setText(s.getPercentToShave());
        numPermutationsTextField.setText(s.getNumPermutations());
        gapToleranceTextField.setText(s.getGapTolerance());
        displayGenesGraphCheckBox.setSelected(s.getIsDisplayGenesGraph());
        displaySamplesGraphCheckBox.setSelected(s.getIsDisplaySamplesGraph());
        displayDataMatrixCheckBox.setSelected(s.getIsDisplayDataMatrix());
        displayShavingCheckBox.setSelected(s.getIsDisplayShaving());
        percentSupervisionTextField.setText(s.getPercentSupervision());
        percentSupervisionTextField.setEnabled(isPerformSupervised());
        classificationFileNameTextField.setText(s.getClassificationFileName());
        classificationFileNameTextField.setEnabled(isPerformSupervised());
        classificationFileChooserButton.setEnabled(isPerformSupervised());
    }

    /**
     * Stores generic portion of configuration
     *
     * @param s  object reference where values are stored
     */
    private void saveGenericConfigSettings(Settings s)
    {
        s.setNumExtClusters(numExtClustersTextField.getText());
        s.setPercentToShave(percentToShaveTextField.getText());
        s.setNumPermutations(numPermutationsTextField.getText());
        s.setGapTolerance(gapToleranceTextField.getText());
        s.setIsDisplayGenesGraph(isDisplayGenesGraph());
        s.setIsDisplaySamplesGraph(isDisplaySamplesGraph());
        s.setIsDisplayDataMatrix(isDisplayDataMatrix());
        s.setIsDisplayShaving(isDisplayShaving());
        s.setIsPerformSupervised(isPerformSupervised());
        s.setPercentSupervision(percentSupervisionTextField.getText());
        s.setClassificationFileName(classificationFileNameTextField.getText());
    }

    /**
     * Loads raw data portion of configuration
     *
     * @param s  object reference containing values to load
     */
    private void loadRawDataConfigSettings(Settings s)
    {
        rawDataFileNameTextField.setText(s.getRawDataFileName());
    }

    /**
     * Stores raw data portion of configuration
     *
     * @param s  object reference where values are stored
     */
    private void saveRawDataConfigSettings(Settings s)
    {
        s.setRawDataFileName(rawDataFileNameTextField.getText());
    }

    /**
     * Loads demo portion of configuration
     *
     * @param s  object reference containing values to load
     */
    private void loadDemoConfigSettings(Settings s)
    {
        numRowsTextField.setText(s.getNumRows());
        numColumnsTextField.setText(s.getNumColumns());
        amplitudeTextField.setText(s.getAmplitude());

        // Model
        {
            Object[] modelItems = getModelItems();
            String selected = s.getModel();

            for (int i = 0; i < modelItems.length; i++)
            {
                if (selected.compareTo(modelItems[i]) == 0)
                {
                    modelComboBox.setSelectedIndex(i);
                    break;
                }
            }
        }

        // Noise
        {
            Object[] noiseItems = getNoiseItems();
            String selected = s.getNoise();

            for (int i = 0; i < noiseItems.length; i++)
            {
                if (selected.compareTo(noiseItems[i]) == 0)
                {
                    noiseComboBox.setSelectedIndex(i);
                    break;
                }
            }
        }

        degreesFreedomTextField.setText(s.getDegreesFreedom());
    }

    /**
     * Stores raw data portion of configuration
     *
     * @param s  object reference where values are stored
     */
    private void saveDemoConfigSettings(Settings s)
    {
        s.setNumRows(numRowsTextField.getText());
        s.setNumColumns(numColumnsTextField.getText());
        s.setAmplitude(amplitudeTextField.getText());
        s.setModel(getModel(false));
        s.setNoise(getNoise());
        s.setDegreesFreedom(degreesFreedomTextField.getText());
    }

    /**
     * Changes the pane in the tabbed pane
     *
     * @param tab  index representing tabs (in order they were added)
     */
    private void selectPane(int tab)
    {
        tabbedPane.setSelectedIndex(tab);
    }

    /**
     * Creates a generic menu item
     *
     * @param keys            resource bundle keys for combo creation
     * @param actionListener  actionListener to be added to the menuitem
     * @return                instance of <code>JMenuItem</code> object
     */
    private JMenuItem createMenuItem(final String[] keys,
                                     ActionListener actionListener)
    {
        int i = 0;

        String label       = getBundleString(keys[i++]);
        String mnemonic    = getBundleString(keys[i++]);
        String description = getBundleString(keys[i++]);
        String toolTipText = getBundleString(keys[i++]);

        return GuiUtil.createMenuItem(label,
                                      mnemonic,
                                      description,
                                      toolTipText,
                                      actionListener);
    }

    /**
     * Determines a given file's extension
     *
     * @param file  <code>File</code> to extract extension from
     * @return      string containing extension (without preceding dot)
     */
    private String getExtension(File file)
    {
        String fileName = file.getName();
        int dotIndex = fileName.lastIndexOf('.');
        String extension = null;

        if ((dotIndex > 0) && (dotIndex < fileName.length() - 1))
            extension = fileName.substring(dotIndex+1).toLowerCase();

        return extension;
    }

    /**
     * Invokes the parent class's termination routine
     *
     * @param reason  error code
     */
    private void invokeTerminate(int reason)
    {
        ((GeneClustFrame)frame).invokeTerminate(reason);
    }

    // =========================================================================
    // LISTENER/ADAPTERS
    // =========================================================================

    /**
     * AboutButtonListener - Responds to action events.
     *
     * @version 0.95 2001/12/12
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class AboutButtonListener implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Displays the about dialog.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
// :TODO: Should desensitize menu option after selection made
            displayAboutDialog(contentPane);
// :TODO: Should resensitize menu option after about dialog closes
        }
    }


    /**
     * ChooseClassificationFileSelectionButtonListener
     *                                             - Responds to action events.
     *
     * @version 0.95 2001/10/17
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class ChooseClassificationFileSelectionButtonListener
      implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Displays a file chooser dialog to select classification file
         * to be used when supervised shaving is enabled.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            displayChooseClassificationFileSelectionDialog(contentPane);
        }
    }


    /**
     * ChooseDataFileSelectionButtonListener - Responds to action events.
     *
     * @version 0.95 2001/02/21
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class ChooseDataFileSelectionButtonListener implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Displays a file chooser dialog to select raw data file to be used.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            displayChooseDataFileSelectionDialog(contentPane);
        }
    }


    /**
     * DisplayButtonListener - Responds to action events.
     *
     * @version 0.95 2001/02/21
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class DisplayButtonListener implements ActionListener
    {
        public DisplayButtonListener (String aStr)
        {
                this.actionStr = aStr;
        }

        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Initiates running a simulation.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            SProcess simulation;
            SimulationDialog simulationDialog;

            simulationDialog = new SimulationDialog(contentPane);
            simulation = createSimulation(actionStr, simulationDialog);

            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

            try
            {
                simulation.run(actionStr);
            }
            catch (Exception e)
            {
                String formatStr = getBundleString(ERR_SIMULATION_FAILED);
                String reason = e.getMessage();

                if (reason == null)
                    reason = getBundleString(NO_EXCEPTION_DETAIL_MESSAGE);

                Object[] args = { reason };
                MessageFormat mf = new MessageFormat(formatStr);
                final String errorMessage = mf.format(args);
                displayErrorDialog(contentPane, errorMessage);
            }
        }

        // =====================================================================
        // PRIVATE INTERFACE
        // =====================================================================

        /**
         * Creates a new simulation based on selected mode.
         *
         * @param simulationDialog  reference to <code>SimulationDialog<code>
         *                          object that displays simulation progress
         * @return                  reference to <code>Simulation<code> object
         */
        private SProcess createSimulation(String actionStr, SimulationDialog simulationDialog)
        {
	    if (actionStr.equals ("display"))
	    {
                return new DisplayProcess(simulationDialog,
					  "X11",
                                          isDisplayGenesGraph(),
                                          isDisplaySamplesGraph(),
                                          isDisplayDataMatrix(),
                                          isDisplayShaving());
	    }
	    if (actionStr.equals ("print"))
	    {
                return new DisplayProcess(simulationDialog,
					  "eps",
                                          isDisplayGenesGraph(),
                                          isDisplaySamplesGraph(),
                                          isDisplayDataMatrix(),
                                          isDisplayShaving());
	    }
	    // if (actionStr.equals ("shave"))
	    {
		if (isDemoEnabled() == true)
		{
                return new DemoSimulation(simulationDialog,
                                          isPerformSupervised(),
                                          getPercentSupervision(),
                                          getClassificationFileName(),
                                          getNumClustersToExtract(),
                                          getPercentToShave(),
                                          getNumPermutations(),
                                          getGapTolerance(),
                                          isWriteMatrix(),
                                          getNumRows(),
                                          getNumColumns(),
                                          getAmplitude(),
                                          getModel(true),
                                          getNoiseIndex(),
                                          getDegreesFreedom());
		}
		else
		{
                return new RawDataSimulation(simulationDialog,
                                             isPerformSupervised(),
                                             getPercentSupervision(),
                                             getClassificationFileName(),
                                             getNumClustersToExtract(),
                                             getPercentToShave(),
                                             getNumPermutations(),
                                             getGapTolerance(),
                                             isWriteMatrix(),
                                             getRawDataFileName());
		}
	    }
        }

        // =========================================================================
        // OBJECT ATTRIBUTES
        // =========================================================================
        private String actionStr;
    }


    /**
     * ExitButtonListener - Responds to action events.
     *
     * @version 0.95 2001/12/17
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class ExitButtonListener implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Terminates the application.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            invokeTerminate(0);
        }
    }


    /**
     * OpenConfigFileSelectionButtonListener - Responds to action events.
     *
     * @version 0.95 2001/12/17
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class OpenConfigFileSelectionButtonListener implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Displays a file chooser dialog and opens user selected file.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            displayOpenConfigFileSelectionDialog(contentPane);
        }
    }


    /**
     * OverviewButtonListener - Responds to action events.
     *
     * @version 0.95 2001/12/12
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class OverviewButtonListener implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Displays the overview dialog frame.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            displayOverviewDialog(contentPane);
        }
    }


    /**
     * PerformSupervisedCheckBoxListener - Responds to action events.
     *
     * @version 0.95 2001/10/17
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class PerformSupervisedCheckBoxListener implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Enables supervised shaving fields on GUI.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            boolean enabled = performSupervisedCheckBox.isSelected();

            classificationFileNameTextField.setEnabled(enabled);
            classificationFileChooserButton.setEnabled(enabled);
            percentSupervisionTextField.setEnabled(enabled);
        }
    }


    /**
     * SaveConfigFileSelectionButtonListener - Responds to action events.
     *
     * @version 0.95 2001/12/17
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class SaveConfigFileSelectionButtonListener implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Displays a file chooser dialog and saves current settings in
         * user selected file.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            displaySaveConfigFileSelectionDialog(contentPane);
        }
    }


    /**
     * UserGuideButtonListener - Responds to action events.
     *
     * @version 0.95 2001/12/12
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class UserGuideButtonListener implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Displays the user guide on the website using a browser.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            String url = getBundleString(USER_GUIDE_HTTP_URL);
            InetProtocolHandler.openURL(url);
        }
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message
     */
    static String getBundleString(String propertyKey)
    {
        return GeneClust.getBundleString(propertyKey);
    }

    // =========================================================================
    // CONSTANTS
    // =========================================================================

    final int UNLIMITED = 0;
    final int RAWDATA_PANE = 0;
    final int DEMO_PANE = 1;

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static ResourceBundle bundle;
    private static Color steelblue;
    private static Color firebrick;
    private static String dataDirName;

    static
    {
        final String keyGCDataDir = "geneclust.data";

        dataDirName = System.getProperty(keyGCDataDir);

        {
            final int[] rgbSteelBlue = {  70, 130, 180 };
            steelblue = new Color(rgbSteelBlue[0],
                                  rgbSteelBlue[1],
                                  rgbSteelBlue[2]);

            final int[] rgbFireBrick = { 178,  34,  34 };
            firebrick = new Color(rgbFireBrick[0],
                                  rgbFireBrick[1],
                                   rgbFireBrick[2]);
        }
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    private Container contentPane;
    private JFrame frame;
    private JMenuBar menuBar;
    private JTabbedPane tabbedPane;
    private JCheckBox displayGraphCheckBox;
    private JCheckBox displayGenesGraphCheckBox;
    private JCheckBox displaySamplesGraphCheckBox;
    private JCheckBox displayDataMatrixCheckBox;
    private JCheckBox displayShavingCheckBox;
    private JCheckBox performSupervisedCheckBox;
    private PathnameTextField classificationFileNameTextField;
    private JButton classificationFileChooserButton;
    private NumericTextField percentSupervisionTextField;
    private NumericTextField numExtClustersTextField;
    private NumericTextField percentToShaveTextField;
    private NumericTextField numPermutationsTextField;
    private NumericTextField gapToleranceTextField;
    private PathnameTextField rawDataFileNameTextField;
    private NumericTextField numRowsTextField;
    private NumericTextField numColumnsTextField;
    private NumericTextField amplitudeTextField;
    private NumericTextField degreesFreedomTextField;
    private JComboBox modelComboBox;
    private JComboBox noiseComboBox;
    private GCAboutDialog aboutDialog;
}

