/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for generic splash screen
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)DefaultSplashScreen.java    1.0 2001/03/22
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JWindow;
import javax.swing.SwingUtilities;


/**
 * DefaultSplashScreen - Creates a generic splash screen for use at
 *                       application startup
 *
 * @version 1.0 2001/03/22
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class DefaultSplashScreen extends JWindow
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor specifying image filename and frame
     *
     * @param imageFileName  string containing image filename to display
     * @param frame          parent component
     */
    public DefaultSplashScreen(String imageFileName, Frame frame)
    {
        this(imageFileName, frame, THREE_SECONDS);
    }

    /**
     * Class constructor specifying image filename, frame, and delay
     *
     * @param imageFileName  string containing image filename to display
     * @param frame          parent component
     * @param waitTime       milliseconds to delay before auto close
     */
    public DefaultSplashScreen(String imageFileName, Frame frame, long waitTime)
    {
        this(new ImageIcon(imageFileName), frame, waitTime);
    }

    /**
     * Class constructor specifying image and frame
     *
     * @param image  image to display
     * @param frame  parent component
     */
    public DefaultSplashScreen(ImageIcon image, Frame frame)
    {
        this(image, frame, THREE_SECONDS);
    }

    /**
     * Class constructor specifying image, frame, and delay
     *
     * @param image     image to display
     * @param frame     parent component
     * @param waitTime  milliseconds to delay before auto close
     */
    public DefaultSplashScreen(ImageIcon image, Frame frame, long waitTime)
    {
        this(new DefaultAboutPanel(image), frame, waitTime);
    }

    /**
     * Class constructor specifying about panel, frame, and delay
     *
     * @param aboutPanel  instance of <code>AboutPanel</code> object
     * @param frame       parent component
     * @param waitTime    milliseconds to delay before auto close
     */
    public DefaultSplashScreen(AboutPanel aboutPanel,
                               Frame frame,
                               long waitTime)
    {
        super(frame);
        setTimeOut(waitTime);
        this.aboutPanel = aboutPanel;
        this.statusPanel = createStatusPanel();
        getContentPane().add((JPanel)aboutPanel, BorderLayout.CENTER);
        getContentPane().add(statusPanel, BorderLayout.SOUTH);
        pack();
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Displays the splashscreen on the screen, centered on its parent
     */
    public void display()
    {
        centerOnScreen();
        addMouseListener(new MouseEventHandler());

        Runnable waitRunner = new WaitRunnable();
        setVisible(true);
        Thread splashThread = new Thread(waitRunner, "SplashThread");
        splashThread.start();
    }

    /**
     * Sets the banner to be displayed
     *
     * @param banner  string specifying banner text to be displayed
     * @see AboutPanel
     */
    public void setBannerLine(String banner)
    {
        aboutPanel.setBannerLine(banner);
    }

    /**
     * Sets the copyright to be displayed
     *
     * @param copyright  string specifying banner text to be displayed
     * @see AboutPanel
     */
    public void setCopyrightLine(String copyright)
    {
        aboutPanel.setCopyrightLine(copyright);
    }

    /**
     * Shows or hides the copyright information depending on the
     * value of parameter
     *
     * @param visible  if true, shows copyright information; otherwise, hides it
     * @see AboutPanel
     */
    public void setCopyrightVisible(boolean visible)
    {
        aboutPanel.setCopyrightVisible(visible);
    }

    /**
     * Sets the legal text to be displayed
     *
     * @param legalities  string specifying legal text to be displayed
     * @see AboutPanel
     */
    public void setLegalitiesLine(String legalities)
    {
        aboutPanel.setLegalitiesLine(legalities);
    }

    /**
     * Shows or hides the legal information depending on the
     * value of parameter
     *
     * @param visible  if true, shows legal information; otherwise, hides it
     * @see AboutPanel
     */
    public void setLegalitiesVisible(boolean visible)
    {
        aboutPanel.setLegalitiesVisible(visible);
    }

    /**
     * Sets the application status text to be displayed
     *
     * @param status  string specifying application status text to be displayed
     */
    public void setStatusLine(String status)
    {
        statusLine.setText(status);
    }

    /**
     * Shows or hides the application status information depending on the
     * value of parameter
     *
     * @param visible  if true, shows legal information; otherwise, hides it
     */
    public void setStatusVisible(boolean visible)
    {
        statusLine.setVisible(visible);
    }

    /**
     * Sets the amount of time to display the splash screen. Must be set
     * prior to display.
     *
     * @param waitTime  amount of time in milliseconds
     */
    public void setTimeOut(long waitTime)
    {
        millisecsToWait = waitTime;
    }

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Centers the splashscreen in the middle of the screen
     */
    protected void centerOnScreen()
    {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        /*
           Dimension windowSize = aboutPanel.getPreferredSize();
         */
        Dimension windowSize = getPreferredSize();

        //System.out.println("windowSize = " + windowSize);
        setLocation(screenSize.width/2 - (windowSize.width/2),
                    screenSize.height/2 - (windowSize.height/2));
    }

    /**
     * Closes the splashscreen and releases all native screen resources
     */
    protected void close()
    {
        setVisible(false);
        dispose();
    }

    /**
     * Creates a pane for displaying status information
     *
     * @return <code>JPanel</code> representing status panel
     */
    protected JPanel createStatusPanel()
    {
        JPanel statusPanel = new JPanel();
        statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.Y_AXIS));
        statusPanel.setBackground(Color.black);

        // Add elements
        {
            Font serif = new Font("Serif", Font.BOLD, 14);
            JLabel label = new JLabel("***Status***");
            label.setForeground(Color.yellow);
            label.setFont(serif);
            statusLine = label;

            statusPanel.add(statusLine);
        }

        return statusPanel;
    }

    // =========================================================================
    // MAIN
    // =========================================================================

    /**
     * Main entry-point for the DefaultSplashScreen class
     *
     * @param argv  string array containing command line arguments
     */
    public static void main(String[] argv)
    {
        final String iconPathName = "acmelogo.gif";

        DefaultSplashScreen splash;

        splash = new DefaultSplashScreen(new ImageIcon(iconPathName),
                                         new JFrame());
        splash.setBannerLine("UnknownApp");
        splash.setCopyrightLine("Copyright 2001, Acme Software");
        splash.setLegalitiesLine("All rights reserved");
        splash.setStatusLine("Starting up...");
        splash.setTimeOut(DefaultSplashScreen.FOUR_SECONDS);
        splash.display();
    }

    // =========================================================================
    // LISTENERS/ADAPTERS
    // =========================================================================

    /**
     * MouseEventHandler - Responds to mouse events.
     *
     * @version 1.0 2001/03/22
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class MouseEventHandler extends MouseAdapter
    {
        // =====================================================================
        // IMPLEMENTS MOUSELISTENER
        // =====================================================================

        /**
         * Invoked when a mouse button has been pressed on a component.
         * Closes the splashscreen.
         *
         * @param event  event which indicates that a mouse action occurred
         */
        public void mousePressed(MouseEvent event)
        {
            close();
        }
    }

    // =========================================================================
    // INNER CLASSES
    // =========================================================================

    /**
     * CloseRunnable - Executed by thread.
     *
     * @version 1.0 2001/03/22
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class CloseRunnable implements Runnable
    {
        // =====================================================================
        // IMPLEMENTS RUNNABLE
        // =====================================================================

        /**
         * Thread entry-point for CloseRunnable object.
         * Closes the splashscreen.
         */
        public void run()
        {
            close();
        }
    }

    /**
     * WaitRunnable - Executed by thread.
     *
     * @version 1.0 2001/03/22
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class WaitRunnable implements Runnable
    {
        // =====================================================================
        // IMPLEMENTS RUNNABLE
        // =====================================================================

        /**
         * Thread entry-point for CloseRunnable object.
         * Provides delay before closing the splashscreen, if user has
         * not already closed it (via mouse click).
         */
        public void run()
        {
            try
            {
                final Runnable closeRunner = new CloseRunnable();

                Thread.sleep(millisecsToWait);
                SwingUtilities.invokeAndWait(closeRunner);
            }
            catch (Exception e)
            {
                e.printStackTrace();
                // can catch InvocationTargetException
                // can catch InterruptedException
            }
        }
    }

    // =========================================================================
    // CONSTANTS
    // =========================================================================

    public static final long ONE_SECOND    = 1000L;
    public static final long TWO_SECONDS   = 2 * ONE_SECOND;
    public static final long THREE_SECONDS = 3 * ONE_SECOND;
    public static final long FOUR_SECONDS  = 4 * ONE_SECOND;
    public static final long FIVE_SECONDS  = 5 * ONE_SECOND;

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    protected AboutPanel aboutPanel;
    protected JPanel statusPanel;
    protected JLabel statusLine;
    private long millisecsToWait = FIVE_SECONDS;
}

