##################### FILE ABSTRACT ############################################
##
## Synopsis::         Source for Raw Data routines
##
## File Name::        %M%
## Version::          %I%
## Type::             source
## Date Completed::   %G%
##
## Environment::
##     Language::     Splus
##     Computer::     Compaq/Alpha
##     OS::           DigitalUNIX
##     OSVersion::    4.0
##
## Module Information::
##     Owner::        MDACC
##     Department::   Biostatistics
##     Researcher::   Kim-Anh Do
##     Application::  geneclust
##
###############################################################################

GCInputFileData.func<-function(fname)
{
    cat("PK_MONITOR_READ_MATRIX\n");

    ## Get proper filename (full path)
    if (is.null(fname) == TRUE || nchar(fname) == 0) {
        stop("No data filename specified\n");
    }
    dataFileName<-GetAbsPathName.func(fname);

    ## Determine the type of file we are dealing with:

    ## find file's extension (must be one of the standard ones)
    ext<-substring(dataFileName,
                   first=nchar(dataFileName)-3,
                   last=nchar(dataFileName));

    flagTSV<-as.logical(charmatch(".tsv",ext,nomatch=-1) == 1);
    flagXLS<-as.logical(charmatch(".xls",ext,nomatch=-1) == 1);

    if (flagTSV == FALSE && flagXLS == FALSE) {
       stop("Unsupported data file type!\n");
    }

    ## Check if file has both col and row labels
    data <- InputData.func (dataFileName);
    SaveData.func (data);

    return (data);
}

GCRawData.func<-function(nclst,percentToShave,nperm,gapTolerance,fname,
                         percentSupervision,classifile)
{
    data <- GCInputFileData.func(fname);
    ## DisplayGraphs.func(data$C, -1, 0);

    numClusters<-ProcessData.func(data, classifile,
                               nclst,
                               percentToShave,
                               nperm,
                               gapTolerance,
                               percentSupervision);
    ## GCDisplayData.func ();

    cat("PK_MONITOR_APP_PROCESSING_COMPLETE\n");
}


##****************************************************************************
InputData.func<-function(dataFileName)
{
    colHeaders<-FALSE;
    rowHeaders<-FALSE;
    fft <- tsvFileFormatTest.C(dataFileName, colHeaders, rowHeaders);
    # print ("FFT: ", quote=F); print (fft);
    hasRowHeaders<-fft[[3]];
    hasColHeaders<-fft[[2]];

    # Read data from file
    if (hasColHeaders == FALSE) {
	print ("Reading table without headers", quote=F);
        A<-read.table(file=dataFileName,sep="\t",header=FALSE,
                      comment.char="",as.is=TRUE,na.strings="NA",skip=0);
    }
    else {
	print ("Reading table with headers", quote=F);
        A<-read.table(file=dataFileName,sep="\t",header=TRUE,
                      comment.char="",as.is=TRUE,na.strings="NA",skip=0);
    }

    # Create matrix from input data
    if (hasRowHeaders == FALSE) {
        B<-A;
    }
    else {
        B<-A[,-1];
	row.names(B) <- A[,1];
    }
    N<-dim(B)[1];
    p<-dim(B)[2];
    C<-matrix(0,nrow=N,ncol=p);

    for (i in 1:p) {
        C[,i]<-cbind(B[,names(B)[i]]);
    }

    cat ("hasRowHeaders=", hasRowHeaders, "\n");
    cat ("hasColHeaders=", hasColHeaders, "\n");
    # No headers are present
    if ((hasRowHeaders == FALSE) && (hasColHeaders == FALSE)) {
        newColLabels<-NULL;
        newRowLabels<-NULL;
    }

    # Column headers are present, no Row headers
    else if ((hasRowHeaders == FALSE) && (hasColHeaders == TRUE)) {
        newColLabels<-names(B);
        newRowLabels<-NULL;
    }

    # Row headers are present, no Column headers
    else if ((hasRowHeaders == TRUE) && (hasColHeaders == FALSE)) {
        newColLabels<-NULL;
        newRowLabels<-row.names(B);
    }

    # Row and Col headers present
    else if ((hasRowHeaders == TRUE) && (hasColHeaders == TRUE)) {
        newColLabels<-names(B);
        newRowLabels<-row.names(B);
    }

    # Validate
    if ((hasColHeaders == TRUE) && (length(newColLabels) != p)) {
        stop("Number of Column labels does not match number of columns in the table\n");
    }
    if ((hasRowHeaders == TRUE) && (length(newRowLabels) != N)) {
        stop("Number of Row labels does not match number of rows in the table\n");
    }

    cat ("newRowLabels=", newRowLabels,"\n");
    return (list(C=C, N=N, p=p, rowLabels=newRowLabels, colLabels=newColLabels));
}

ProcessData.func<-function(data,
                           classfile,
                           nclst,
                           percentToShave,
                           nperm,
                           gapTolerance,
                           percentSupervision)
{

    # Perform geneshaving
    generateLabels<-FALSE;
    nclstAct<-CallGeneShave.func(data$C,
                                 data$N,
                                 data$p,
                                 classfile,
                                 nclst,
                                 percentToShave,
                                 nperm,
                                 gapTolerance,
                                 generateLabels,
                                 percentSupervision);

    cat ("CallGeneShave.func completed:", nclstAct, "clusters generated\n");
    return(nclstAct);
}


#************************************************************************
CallGeneShave.func<-function(C,N,p,classfile,nclst,percentToShave,nperm,
                             gapTolerance,generateLabels,percentSupervision)
{
    w<-as.vector(matrix(t(C),N*p,1));
    is.vector(w);
    ## print ("w:", quote=F);print(w);

    verbose<-TRUE;
    if (is.null(classfile) == FALSE) {
        classFileName<-GetAbsPathName.func(classfile);
    }
    else {
        classFileName<-NULL;
    }
    nclstFound<-32767 ## Value is arbitrary, probably better to use 0
    nclstAct<-GeneShave.C(GCoutput,
    			  nclst,
                          percentToShave,
                          nperm,
                          gapTolerance,
                          w,
                          N,
                          p,
                          verbose = TRUE,
                          generateLabels,
                          writeMatrix = FALSE,
                          percentSupervision,
                          classFileName,
                          nclstFound);

    return(nclstAct);
}

