/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for S-plus only routines
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     ANSI C
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  geneclust
 *
 *============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include "_bool.h"
#include "GeneClust_types.h"
#include "FileFormatTest.h"
#include "splus.h"


/*
 * Private Routine Declarations
 */
static bool IsDouble (const char *);


/*
 * Public Routines
 */

/*============================================================================
 *
 * Name:: tsvFileFormatTest
 *
 * Description::
 *     Reads a tab-separated values data file to (hopefully) determine whether
 *     the data contains row and/or column headers. Data values must be
 *     double values and column headers must be alphanumeric.
 *     Used by S-plus GCRawData.func().
 *
 * Inputs::
 *     tsvPathName   - Pointer to string containing path of datafile.
 *
 * Outputs::
 *     hasColHeaders - Pointer to logical indicating existence of column headers
 *     hasRowHeaders - Pointer to logical indicating existence of row headers
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

void tsvFileFormatTest (
  SCharacter_t *tsvPathName,
  SLogical_t *hasColHeaders,
  SLogical_t *hasRowHeaders)
{
    FILE *tsvFile;
    const char *tsvFileName;

    assert (tsvPathName != NULL);
    assert (hasColHeaders != NULL);
    assert (hasRowHeaders != NULL);

    tsvFileName = *tsvPathName;
    *hasColHeaders = false;
    *hasRowHeaders = false;

    /* Open input file */
    if ((tsvFile = fopen (tsvFileName, "r")) != NULL)
    {
        Buffer_t buffer;

        if (fgets (buffer, sizeof(Buffer_t), tsvFile) != NULL)
        {
            char *token;
            char *newline;

            /* Check the value in the second column of the first line */
            strtok (buffer, "\t\n\0");
            token = strtok (NULL, "\t\n\0");
            if (IsDouble (token) == false)
                *hasColHeaders = true;

            /* Read rest of line, but don't bother to examine it */
            while ((newline = strchr (buffer, '\n')) == NULL)
                fgets (buffer, sizeof(buffer), tsvFile);

            if (fgets (buffer, sizeof(buffer), tsvFile) != NULL)
            {
                /* Check the value in the first column of the second line */
                token = strtok (buffer, "\t\n\0");
                if (IsDouble (token) == false)
                    *hasRowHeaders = true;
            }
        }
        fclose (tsvFile);

#ifdef DEBUG
        printf ("hasColHeaders = %ld,\thasRowHeaders = %ld\n",
                *hasColHeaders, *hasRowHeaders);
#endif
    }
    else {
        fprintf (stderr, "Cannot open <%s> for reading - %s.\n",
                 tsvFileName, strerror (errno));
    }
}


/*
 * Private Routines
 */

/*============================================================================
 *
 * Name:: IsDouble
 *
 * Description::
 *     Attempts to extract a double value from the input string.
 *
 * Inputs::
 *     token - String containing value to be examined
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     Returns true if double value was able to be extracted; otherwise, false.
 *
 *============================================================================*/

static bool IsDouble (const char *token)
{
    double value;
    int numConverted;
    char trash[32];

    assert (token != NULL);

    trash[0] = '\0';
    numConverted = sscanf (token, "%lg%s", &value, trash);
    if (numConverted != 1)
        return false;

    return true;
}

